{-
    The MIT License
    
    Copyright (c) 2010 Korcan Hussein
    
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:
    
    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.
    
    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
    THE SOFTWARE.
-}
{-# LANGUAGE GeneralizedNewtypeDeriving #-}
module GameEnv
(
    AppEnv(),
    runApp,
    getPTurnCell,
    modifyPlayerCount,
    screenWidth,
    screenHeight,
    sceenBpp
) where

import Control.Monad.State
import Control.Monad.Reader

import Grid
import GameState

screenWidth, screenHeight, sceenBpp :: Int
screenWidth  = 640
screenHeight = 480
sceenBpp     = 32

type AppState = StateT AppData IO
newtype AppEnv a = AppEnv { runEnv :: ReaderT AppConfig AppState a }
    deriving (Monad, MonadIO, MonadReader AppConfig, MonadState AppData)

runApp :: AppConfig -> AppData -> AppEnv a -> IO (a, AppData)
runApp config state (AppEnv env) = (runStateT . runReaderT env) config state

getPTurnCell :: AppEnv Cell
getPTurnCell = getTurn >>= mapTurn
 where mapTurn :: PlayerTurn -> AppEnv TileType
       mapTurn Player = getPlayerType
       mapTurn Ai = swapCell `liftM` getPlayerType

modifyPlayerCount :: (Integer -> Integer) -> AppEnv ()
modifyPlayerCount fn = liftM mapTurn getTurn >>= modifyStats
 where mapTurn :: PlayerTurn -> Stats -> Stats
       mapTurn Player = \s@Stats { playerCount=pc } -> s { playerCount=fn pc }
       mapTurn Ai     = \s@Stats { aiCount=pc } -> s { aiCount=fn pc }